import cmd2
from rich.console import Console
from rich.table import Table

from moonshot.api import api_delete_prompt_template, api_get_all_prompt_template_detail
from moonshot.src.utils.find_feature import find_keyword

console = Console()


# ------------------------------------------------------------------------------
# CLI Functions
# ------------------------------------------------------------------------------
def list_prompt_templates(args) -> list | None:
    """
    List all prompt templates available.

    Args:
        args: A namespace object from argparse. It should have an optional attribute:
        find (str): Optional field to find prompt template(s) with a keyword.

    Returns:
        list | None: A list of PromptTemplate or None if there is no result.
    """
    try:
        prompt_templates_list = api_get_all_prompt_template_detail()
        keyword = args.find.lower() if args.find else ""
        if keyword:
            filtered_prompt_templates_list = find_keyword(
                keyword, prompt_templates_list
            )
            if filtered_prompt_templates_list:
                display_prompt_templates(filtered_prompt_templates_list)
                return filtered_prompt_templates_list
            else:
                print("No prompt templates containing keyword found.")
                return None
        else:
            display_prompt_templates(prompt_templates_list)
            return prompt_templates_list
    except Exception as e:
        print(f"[list_prompt_templates]: {str(e)}")


def delete_prompt_template(args) -> None:
    """
    Deletes a prompt_template after confirming with the user.

    Args:
        args (object): The arguments object. It should have a 'prompt_template' attribute
                       which is the ID of the prompt template to delete.
    """
    # Confirm with the user before deleting a prompt template
    confirmation = console.input(
        "[bold red]Are you sure you want to delete the prompt template (y/N)? [/]"
    )
    if confirmation.lower() != "y":
        console.print("[bold yellow]Prompt template deletion cancelled.[/]")
        return
    try:
        api_delete_prompt_template(args.prompt_template)
        print("[delete_prompt_template]: Prompt template deleted.")
    except Exception as e:
        print(f"[delete_prompt_template]: {str(e)}")


# ------------------------------------------------------------------------------
# Helper functions: Display on cli
# ------------------------------------------------------------------------------
def display_prompt_templates(prompt_templates) -> None:
    """
    Display the list of prompt templates in a formatted table.

    This function takes a list of prompt templates and displays them in a formatted table.
    Each row in the table represents a prompt template with its ID, name, description, and contents.
    If the list of prompt templates is empty, it prints a message indicating that no prompt templates were found.

    Args:
        prompt_templates (list): A list of dictionaries, each representing a prompt template.
    """
    table = Table(
        title="List of Prompt Templates",
        show_lines=True,
        expand=True,
        header_style="bold",
    )
    table.add_column("No.", width=2)
    table.add_column("Prompt Template", justify="left", width=50)
    table.add_column("Contains", justify="left", width=48, overflow="fold")
    if prompt_templates:
        for prompt_index, prompt_template in enumerate(prompt_templates, 1):
            (
                id,
                name,
                description,
                contents,
            ) = prompt_template.values()

            prompt_info = f"[red]id: {id}[/red]\n\n[blue]{name}[/blue]\n{description}"
            table.add_section()
            table.add_row(str(prompt_index), prompt_info, contents)
        console.print(table)
    else:
        console.print("[red]There are no prompt templates found.[/red]")


# Delete prompt template arguments
delete_prompt_template_args = cmd2.Cmd2ArgumentParser(
    description="Delete a prompt template.",
    epilog="Example:\n delete_prompt_template squad-shifts",
)

delete_prompt_template_args.add_argument(
    "prompt_template", type=str, help="The ID of the prompt template to delete"
)

# List prompt template arguments
list_prompt_templates_args = cmd2.Cmd2ArgumentParser(
    description="List all prompt templates.",
    epilog='Example:\n list_prompt_templates -f "toxicity"',
)

list_prompt_templates_args.add_argument(
    "-f",
    "--find",
    type=str,
    help="Optional field to find prompt template(s) with keyword",
    nargs="?",
)



import cmd2
from rich.console import Console
from rich.table import Table

from moonshot.api import (
    api_delete_dataset,
    api_get_all_datasets,
    api_get_all_datasets_name,
)
from moonshot.integrations.cli.common.display_helper import display_view_str_format
from moonshot.src.utils.find_feature import find_keyword

console = Console()


# ------------------------------------------------------------------------------
# CLI Functions
# ------------------------------------------------------------------------------
def list_datasets(args) -> list | None:
    """
    List all available datasets.

    This function retrieves all available datasets by calling the api_get_all_datasets function from the
    moonshot.api module. It then displays the datasets using the display_datasets function. If an exception occurs,
    it prints an error message.

    Args:
        args: A namespace object from argparse. It should have an optional attribute:
        find (str): Optional field to find dataset(s) with a keyword.

    Returns:
        list | None: A list of Dataset or None if there is no result.
    """
    try:
        print("Listing datasets may take a while...")
        datasets_list = api_get_all_datasets()
        keyword = args.find.lower() if args.find else ""
        if keyword:
            filtered_datasets_list = find_keyword(keyword, datasets_list)
            if filtered_datasets_list:
                display_datasets(filtered_datasets_list)
                return filtered_datasets_list
            else:
                print("No datasets containing keyword found.")
                return None
        else:
            display_datasets(datasets_list)
            return datasets_list
    except Exception as e:
        print(f"[list_datasets]: {str(e)}")


def view_dataset(args) -> None:
    """
    View a specific dataset.

    This function retrieves all available datasets and their names by calling the api_get_all_datasets and
    api_get_all_datasets_name functions. It then finds the dataset with the name specified in args.dataset_filename
    and displays it using the display_datasets function. If an exception occurs, it prints an error message.

    Args:
        args: A namespace object from argparse. It should have the following attribute:
            dataset_filename (str): The name of the dataset to view.

    Returns:
        None
    """
    try:
        print("Viewing datasets may take a while...")
        datasets_list = api_get_all_datasets()
        datasets_name_list = api_get_all_datasets_name()

        # Find the index of the dataset with the name args.dataset_filename
        dataset_index = datasets_name_list.index(args.dataset_filename)
        # Pass the corresponding dataset from datasets_list to display_datasets
        display_datasets([datasets_list[dataset_index]])

    except Exception as e:
        print(f"[view_dataset]: {str(e)}")


def delete_dataset(args) -> None:
    """
    Delete a dataset.

    This function deletes a dataset with the specified name. It prompts the user for confirmation before proceeding
    with the deletion. If the user confirms, it calls the api_delete_dataset function from the moonshot.api module to
    delete the dataset. If the deletion is successful, it prints a confirmation message. If an exception occurs, it
    prints an error message.

    Args:
        args: A namespace object from argparse. It should have the following attribute:
            dataset_name (str): The name of the dataset to delete.

    Returns:
        None
    """
    # Confirm with the user before deleting a dataset
    confirmation = console.input(
        "[bold red]Are you sure you want to delete the dataset (y/N)? [/]"
    )
    if confirmation.lower() != "y":
        console.print("[bold yellow]Dataset deletion cancelled.[/]")
        return
    try:
        api_delete_dataset(args.dataset)
        print("[delete_dataset]: Dataset deleted.")
    except Exception as e:
        print(f"[delete_dataset]: {str(e)}")


# ------------------------------------------------------------------------------
# Helper functions: Display on cli
# ------------------------------------------------------------------------------
def display_datasets(datasets_list: list):
    """
    Displays a list of datasets in a table format.

    This function takes a list of datasets and displays them in a table format with each dataset's name, description,
    and other relevant details. If the list is empty, it prints a message indicating that no datasets are found.

    Args:
        datasets_list (list): A list of dictionaries, where each dictionary contains the details of a dataset.

    Returns:
        None
    """
    if datasets_list:
        table = Table(
            title="List of Datasets", show_lines=True, expand=True, header_style="bold"
        )
        table.add_column("No.", width=2)
        table.add_column("Dataset", justify="left", width=78)
        for dataset_no, dataset in enumerate(datasets_list, 1):
            (
                id,
                name,
                description,
                _,
                num_of_dataset_prompts,
                created_date,
                reference,
                license,
            ) = dataset.values()

            prompt_info = display_view_str_format("Prompts", num_of_dataset_prompts)
            created_date_info = display_view_str_format("Created Date", created_date)
            license_info = display_view_str_format("License", license)
            reference_info = display_view_str_format("Reference", reference)

            dataset_info = (
                f"[red]{id}[/red]\n\n[blue]{name}[/blue]\n{description}\n\n"
                f"{prompt_info}\n\n{created_date_info}\n\n{license_info}\n\n{reference_info}"
            )

            table.add_section()
            table.add_row(str(dataset_no), dataset_info)
        console.print(table)
    else:
        console.print("[red]There are no datasets found.[/red]")


# ------------------------------------------------------------------------------
# Cmd2 Arguments Parsers
# ------------------------------------------------------------------------------
# View dataset arguments
view_dataset_args = cmd2.Cmd2ArgumentParser(
    description="View a dataset file.",
    epilog="Example:\n view_dataset bbq-lite-age-ambiguous",
)
view_dataset_args.add_argument(
    "dataset_filename", type=str, help="Name of the dataset file"
)

# Delete dataset arguments
delete_dataset_args = cmd2.Cmd2ArgumentParser(
    description="Delete a dataset.",
    epilog="Example:\n delete_dataset bbq-lite-age-ambiguous",
)
delete_dataset_args.add_argument("dataset", type=str, help="Name of the dataset")

# List dataset arguments
list_datasets_args = cmd2.Cmd2ArgumentParser(
    description="List all datasets.",
    epilog='Example:\n list_datasets -f "bbq"',
)

list_datasets_args.add_argument(
    "-f",
    "--find",
    type=str,
    help="Optional field to find dataset(s) with keyword",
    nargs="?",
)



from pydantic import validate_call

from moonshot.src.datasets.dataset import Dataset


# ------------------------------------------------------------------------------
# Datasets APIs
# ------------------------------------------------------------------------------
@validate_call
def api_delete_dataset(ds_id: str) -> bool:
    """
    Deletes a dataset identified by its unique dataset ID.

    Args:
        ds_id (str): The unique identifier for the dataset to be deleted.

    Returns:
        bool: True if the dataset was successfully deleted.

    Raises:
        Exception: If the deletion process encounters an error.
    """
    return Dataset.delete(ds_id)


def api_get_all_datasets() -> list[dict]:
    """
    This function retrieves all available datasets and returns them as a list of dictionaries. Each dictionary
    represents a result and contains its information.

    Returns:
        list[dict]: A list of dictionaries, each representing a result.
    """
    _, datasets = Dataset.get_available_items()
    return [dataset.to_dict() for dataset in datasets]


def api_get_all_datasets_name() -> list[str]:
    """
    This function retrieves all available datasets names and returns them as a list.

    Returns:
        list[str]: A list of datasets names.
    """
    datasets_name, _ = Dataset.get_available_items()
    return datasets_name



from .... import api as moonshot_api
from ..schemas.dataset_response_dto import DatasetResponseDTO
from ..services.base_service import BaseService
from ..services.utils.exceptions_handler import exception_handler


class DatasetService(BaseService):
    @exception_handler
    def get_all_datasets(self) -> list[DatasetResponseDTO]:
        datasets = moonshot_api.api_get_all_datasets()
        return [DatasetResponseDTO.model_validate(dataset) for dataset in datasets]

    @exception_handler
    def get_all_datasets_name(self) -> list[str] | None:
        datasets = moonshot_api.api_get_all_datasets_name()
        return datasets

    @exception_handler
    def delete_dataset(self, dataset_id: str) -> None:
        moonshot_api.api_delete_dataset(dataset_id)



import cmd2
from rich.console import Console
from rich.table import Table

from moonshot.api import api_get_all_run
from moonshot.integrations.cli.common.display_helper import (
    display_view_list_format,
    display_view_str_format,
)
from moonshot.src.utils.find_feature import find_keyword

console = Console()


# ------------------------------------------------------------------------------
# CLI Functions
# ------------------------------------------------------------------------------
def list_runs(args) -> list | None:
    """
    List all runs.

    This function retrieves all available runs by calling the api_get_all_run function from the
    moonshot.api module. It then calls the display_runs function to present the retrieved run information
    in a user-friendly format on the command line interface. If an exception occurs during the retrieval
    or display process, it prints an error message.

    Args:
        args: A namespace object from argparse. It should have an optional attribute:
        find (str): Optional field to find run(s) with a keyword.

    Returns:
        list | None: A list of Run or None if there is no result.
    """
    try:
        runner_run_info = api_get_all_run()
        keyword = args.find.lower() if args.find else ""
        if keyword:
            filtered_runs_list = find_keyword(keyword, runner_run_info)
            if filtered_runs_list:
                display_runs(filtered_runs_list)
                return filtered_runs_list
            else:
                print("No runs containing keyword found.")
                return None
        else:
            display_runs(runner_run_info)
            return runner_run_info
    except Exception as e:
        print(f"[list_runs]: {str(e)}")


def view_run(args) -> None:
    """
    View the details of a specific run.

    This function retrieves and displays information about a specific run associated with a runner. It uses the runner
    identifier provided in the arguments to fetch the data and then calls the display_runs function to present it in a
    user-friendly format.

    Args:
        args: A namespace object from argparse. It should have the following attribute:
            runner (str): The identifier of the runner whose runs are to be viewed.

    Returns:
        None
    """
    try:
        runner_run_info = api_get_all_run(args.runner_id)
        display_runs(runner_run_info)
    except Exception as e:
        print(f"[view_run]: {str(e)}")


# ------------------------------------------------------------------------------
# Helper functions: Display on cli
# ------------------------------------------------------------------------------
def display_runs(runs_list: list):
    """
    Display a list of runs in a table format.

    This function takes a list of run information and displays it in a table format using the rich library's
    Table object.

    Each run's details are formatted and added as a row in the table.
    If there are no runs to display, a message is printed to indicate that no results were found.

    Args:
        runs_list (list): A list of dictionaries, where each dictionary contains details of a run.

    Returns:
        None
    """
    if runs_list:
        table = Table(
            title="List of Runs", show_lines=True, expand=True, header_style="bold"
        )
        table.add_column("No.", width=2)
        table.add_column("Run", justify="left", width=78)
        table.add_column("Contains", justify="left", width=20, overflow="fold")
        for run_number, run in enumerate(runs_list, 1):
            (
                run_id,
                runner_id,
                runner_type,
                runner_args,
                endpoints,
                results_file,
                start_time,
                end_time,
                duration,
                error_messages,
                raw_results,
                results,
                status,
            ) = run.values()

            duration_info = (
                f"[blue]Period:[/blue] {start_time} - {end_time} ({duration}s)"
            )
            run_id = display_view_str_format("Run ID", run_id)
            runner_id = display_view_str_format("Runner ID", runner_id)
            runner_type = display_view_str_format("Runner Type", runner_type)
            runner_args = display_view_str_format("Runner Args", runner_args)
            status_info = display_view_str_format("Status", status)
            results_info = display_view_str_format("Results File", results_file)
            endpoints_info = display_view_list_format("Endpoints", endpoints)
            error_messages_info = display_view_list_format(
                "Error Messages", error_messages
            )

            has_raw_results = bool(raw_results)
            has_results = bool(results)

            result_info = f"[red]{runner_id}[/red]\n\n{run_id}\n\n{duration_info}\n\n{status_info}"
            contains_info = (
                f"{results_info}\n\n{error_messages_info}\n\n{endpoints_info}\n\n"
                f"[blue]Has Raw Results: {has_raw_results}[/blue]\n\n"
                f"[blue]Has Results: {has_results}[/blue]"
            )

            table.add_section()
            table.add_row(str(run_number), result_info, contains_info)
        console.print(table)
    else:
        console.print("[red]There are no results found.[/red]")


# ------------------------------------------------------------------------------
# Cmd2 Arguments Parsers
# ------------------------------------------------------------------------------
# View run arguments
view_run_args = cmd2.Cmd2ArgumentParser(
    description="View a runner runs.",
    epilog="Example:\n view_run my-new-cookbook-runner",
)
view_run_args.add_argument("runner_id", type=str, help="Name of the runner")


# List run arguments
list_runs_args = cmd2.Cmd2ArgumentParser(
    description="List all runs.",
    epilog='Example:\n list_runs -f "my-run"',
)

list_runs_args.add_argument(
    "-f",
    "--find",
    type=str,
    help="Optional field to find run(s) with keyword",
    nargs="?",
)



from ast import literal_eval

import cmd2
from rich.console import Console
from rich.markup import escape
from rich.table import Table

from moonshot.api import (
    api_create_endpoint,
    api_delete_endpoint,
    api_get_all_connector_type,
    api_get_all_endpoint,
    api_read_endpoint,
    api_update_endpoint,
)
from moonshot.src.utils.find_feature import find_keyword

console = Console()


# ------------------------------------------------------------------------------
# CLI Functions
# ------------------------------------------------------------------------------
def add_endpoint(args) -> None:
    """
    Add a new endpoint.

    This function adds a new endpoint by calling the api_create_endpoint function from the
    moonshot.api module using the endpoint details provided in the args.

    Args:
        args: A namespace object from argparse. It should have the following attributes:
            name (str): The name of the new endpoint.
            connector_type (str): The type of the connector (e.g., 'GPT-3', 'Bert', etc.).
            uri (str): The URI for the connector's API.
            token (str): The access token for the API.
            max_calls_per_second (int): The maximum number of API calls allowed per second.
            max_concurrency (int): The maximum number of concurrent API calls.
            params (str): A string representation of a dictionary containing additional parameters for the connector.

    Returns:
        None
    """
    try:
        params_dict = literal_eval(args.params)

        new_endpoint_id = api_create_endpoint(
            args.name,
            args.connector_type,
            args.uri,
            args.token,
            args.max_calls_per_second,
            args.max_concurrency,
            params_dict,
        )
        print(f"[add_endpoint]: Endpoint ({new_endpoint_id}) created.")
    except Exception as e:
        print(f"[add_endpoint]: {str(e)}")


def list_endpoints(args) -> list | None:
    """
    List all endpoints.

    This function retrieves all endpoints by calling the api_get_all_endpoint function from the
    moonshot.api module. It then displays the endpoints using the display_endpoints function.

    Args:
        args: A namespace object from argparse. It should have an optional attribute:
        find (str): Optional field to find endpoint(s) with a keyword.

    Returns:
        list | None: A list of ConnectorEndpoint or None if there is no result.
    """
    try:
        endpoints_list = api_get_all_endpoint()
        keyword = args.find.lower() if args.find else ""
        if keyword:
            filtered_endpoints_list = find_keyword(keyword, endpoints_list)
            if filtered_endpoints_list:
                display_endpoints(filtered_endpoints_list)
                return filtered_endpoints_list
            else:
                print("No endpoints containing keyword found.")
                return None
        else:
            display_endpoints(endpoints_list)
            return endpoints_list
    except Exception as e:
        print(f"[list_endpoints]: {str(e)}")


def list_connector_types(args) -> list | None:
    """
    List all connector types.

    This function retrieves all connector types by calling the api_get_all_connector_type function from the
    moonshot.api module. It then displays the connector types using the display_connector_types function.

    Args:
        args: A namespace object from argparse. It should have an optional attribute:
        find (str): Optional field to find connector type(s) with a keyword.

    Returns:
        list | None: A list of Connector or None if there is no result.
    """
    try:
        connector_type_list = api_get_all_connector_type()
        keyword = args.find.lower() if args.find else ""
        if keyword:
            filtered_connector_type_list = find_keyword(keyword, connector_type_list)
            if filtered_connector_type_list:
                display_connector_types(filtered_connector_type_list)
                return filtered_connector_type_list
            else:
                print("No connectors containing keyword found.")
                return None
        else:
            display_connector_types(connector_type_list)
            return connector_type_list
    except Exception as e:
        print(f"[list_connector_types]: {str(e)}")


def view_endpoint(args) -> None:
    """
    View a specific endpoint.

    This function retrieves a specific endpoint by calling the api_read_endpoint function from the
    moonshot.api module using the endpoint name provided in the args. It then displays the endpoint
    information using the display_endpoints function.

    Args:
        args: A namespace object from argparse. It should have the following attribute:
            endpoint (str): The name of the endpoint to view.

    Returns:
        None
    """
    try:
        endpoint_info = api_read_endpoint(args.endpoint)
        display_endpoints([endpoint_info])
    except Exception as e:
        print(f"[view_endpoint]: {str(e)}")


def update_endpoint(args) -> None:
    """
    Update a specific endpoint.

    This function updates a specific endpoint by calling the api_update_endpoint function from the
    moonshot.api module using the endpoint name and update values provided in the args.

    Args:
        args: A namespace object from argparse. It should have the following attributes:
            endpoint (str): The name of the endpoint to update.
            update_kwargs (str): A string representation of a dictionary. Each key-value pair in the dictionary
            represents a field to update in the endpoint and the new value for that field.

    Returns:
        None
    """
    try:
        endpoint = args.endpoint
        update_values = dict(literal_eval(args.update_kwargs))
        api_update_endpoint(endpoint, **update_values)
        print("[update_endpoint]: Endpoint updated.")
    except Exception as e:
        print(f"[update_endpoint]: {str(e)}")


def delete_endpoint(args) -> None:
    """
    Delete a specific endpoint.

    This function deletes a specific endpoint by calling the api_delete_endpoint function from the
    moonshot.api module using the endpoint name provided in the args. Before deletion, it asks for
    user confirmation. If the user confirms, the endpoint is deleted; otherwise, the deletion is cancelled.

    Args:
        args: A namespace object from argparse. It should have the following attribute:
            endpoint (str): The name of the endpoint to delete.

    Returns:
        None
    """
    # Confirm with the user before deleting an endpoint
    confirmation = console.input(
        "[bold red]Are you sure you want to delete the endpoint (y/N)? [/]"
    )
    if confirmation.lower() != "y":
        console.print("[bold yellow]Endpoint deletion cancelled.[/]")
        return
    try:
        api_delete_endpoint(args.endpoint)
        print("[delete_endpoint]: Endpoint deleted.")
    except Exception as e:
        print(f"[delete_endpoint]: {str(e)}")


# ------------------------------------------------------------------------------
# Helper functions: Display on cli
# ------------------------------------------------------------------------------
def display_connector_types(connector_types: list) -> None:
    """
    Display a list of connector types.

    This function takes a list of connector types and displays them in a table format. If the list is empty, it prints a
    message indicating that no connector types were found.

    Args:
        connector_types (list): A list of connector types.

    Returns:
        None
    """
    if connector_types:
        table = Table(
            title="List of Connector Types",
            show_lines=True,
            expand=True,
            header_style="bold",
        )
        table.add_column("No.", width=2)
        table.add_column("Connector Type", justify="left", width=78)
        for connector_id, connector_type in enumerate(connector_types, 1):
            table.add_section()
            table.add_row(str(connector_id), connector_type)
        console.print(table)
    else:
        console.print("[red]There are no connector types found.[/red]")


def display_endpoints(endpoints_list):
    """
    Display a list of endpoints.

    This function takes a list of endpoints and displays them in a table format. If the list is empty, it prints a
    message indicating that no endpoints were found.

    Args:
        endpoints_list (list): A list of endpoints. Each endpoint is a dictionary with keys 'id', 'name',
        'connector_type', 'uri', 'token', 'max_calls_per_second', 'max_concurrency', 'params', and 'created_date'.

    Returns:
        None
    """
    if endpoints_list:
        table = Table(
            title="List of Connector Endpoints",
            show_lines=True,
            expand=True,
            header_style="bold",
        )
        table.add_column("No.", justify="left", width=2)
        table.add_column("Id", justify="left", width=10)
        table.add_column("Name", justify="left", width=10)
        table.add_column("Connector Type", justify="left", width=10)
        table.add_column("Uri", justify="left", width=10)
        table.add_column("Token", justify="left", width=10)
        table.add_column("Max Calls Per Second", justify="left", width=5)
        table.add_column("Max concurrency", justify="left", width=5)
        table.add_column("Params", justify="left", width=30)
        table.add_column("Created Date", justify="left", width=8)

        for endpoint_id, endpoint in enumerate(endpoints_list, 1):
            (
                id,
                name,
                connector_type,
                uri,
                token,
                max_calls_per_second,
                max_concurrency,
                params,
                created_date,
            ) = endpoint.values()
            table.add_section()
            table.add_row(
                str(endpoint_id),
                id,
                name,
                connector_type,
                uri,
                token,
                str(max_calls_per_second),
                str(max_concurrency),
                escape(str(params)),
                created_date,
            )
        console.print(table)
    else:
        console.print("[red]There are no endpoints found.[/red]")


# ------------------------------------------------------------------------------
# Cmd2 Arguments Parsers
# ------------------------------------------------------------------------------
# Add endpoint arguments
add_endpoint_args = cmd2.Cmd2ArgumentParser(
    description="Add a new endpoint. The 'name' argument will be slugified to create a unique identifier.",
    epilog="Example:\n add_endpoint openai-connector 'OpenAI GPT3.5 Turbo 1106' "
    "MY_URI ADD_YOUR_TOKEN_HERE 1 1 \"{'temperature': 0.5, 'model': 'gpt-3.5-turbo-1106'}\"",
)
add_endpoint_args.add_argument(
    "connector_type",
    type=str,
    help="Type of connection for the endpoint",
)
add_endpoint_args.add_argument("name", type=str, help="Name of the new endpoint")
add_endpoint_args.add_argument("uri", type=str, help="URI of the new endpoint")
add_endpoint_args.add_argument("token", type=str, help="Token of the new endpoint")
add_endpoint_args.add_argument(
    "max_calls_per_second",
    type=int,
    help="Max calls per second of the new endpoint",
)
add_endpoint_args.add_argument(
    "max_concurrency", type=int, help="Max concurrency of the new endpoint"
)
add_endpoint_args.add_argument("params", type=str, help="Params of the new endpoint")

# Update endpoint arguments
update_endpoint_args = cmd2.Cmd2ArgumentParser(
    description="Update an endpoint.",
    epilog=(
        "Available keys:\n"
        "  name: name of the endpoint\n"
        "  uri: URI of the endpoint\n"
        "  token: Token of the endpoint\n"
        "  max_calls_per_second: Rate limit for max calls per second\n"
        "  max_concurrency: Rate limit for max concurrency\n"
        "  params: Extra arguments for the endpoint\n\n"
        "Example:\n"
        "  update_endpoint openai-gpt4 \"[('name', 'my-special-openai-endpoint'), "
        "('uri', 'my-uri-loc'), ('token', 'my-token-here')]\""
    ),
)
update_endpoint_args.add_argument(
    "endpoint",
    type=str,
    help="ID of the endpoint. This field is not editable via CLI after creation.",
)
update_endpoint_args.add_argument(
    "update_kwargs", type=str, help="Update endpoint key/value"
)

# View endpoint arguments
view_endpoint_args = cmd2.Cmd2ArgumentParser(
    description="View an endpoint.",
    epilog="Example:\n view_endpoint openai-gpt4",
)
view_endpoint_args.add_argument("endpoint", type=str, help="ID of the endpoint")

# Delete endpoint arguments
delete_endpoint_args = cmd2.Cmd2ArgumentParser(
    description="Delete an endpoint.",
    epilog="Example:\n delete_endpoint openai-gpt4",
)
delete_endpoint_args.add_argument("endpoint", type=str, help="ID of the endpoint")

# List endpoint arguments
list_endpoints_args = cmd2.Cmd2ArgumentParser(
    description="List all endpoints.",
    epilog='Example:\n list_endpoints -f "gpt"',
)

list_endpoints_args.add_argument(
    "-f",
    "--find",
    type=str,
    help="Optional field to find endpoint(s) with keyword",
    nargs="?",
)


# List connector types arguments
list_connector_types_args = cmd2.Cmd2ArgumentParser(
    description="List all connector types.",
    epilog='Example:\n list_connector_types -f "openai"',
)

list_connector_types_args.add_argument(
    "-f",
    "--find",
    type=str,
    help="Optional field to find connector type(s) with keyword",
    nargs="?",
)



from datetime import datetime

from moonshot.src.bookmark.bookmark_arguments import BookmarkArguments
from moonshot.src.configs.env_variables import EnvVariables
from moonshot.src.storage.storage import Storage
from moonshot.src.utils.log import configure_logger

# Create a logger for this module
logger = configure_logger(__name__)


class Bookmark:
    _instance = None

    def __new__(cls, db_name="bookmark"):
        """
        Create a new instance of the Bookmark class or return the existing instance.

        Args:
            db_name (str): The name of the database.

        Returns:
            Bookmark: The singleton instance of the Bookmark class.
        """
        if cls._instance is None:
            cls._instance = super(Bookmark, cls).__new__(cls)
            cls._instance.__init_instance(db_name)
        return cls._instance

    @classmethod
    def get_instance(cls, db_name="bookmark"):
        """
        Get the singleton instance of the Bookmark class.

        Args:
            db_name (str): The name of the database.

        Returns:
            Bookmark: The singleton instance of the Bookmark class.
        """
        if cls._instance is None:
            cls._instance = super(Bookmark, cls).__new__(cls)
            cls._instance.__init_instance(db_name)
        return cls._instance

    sql_create_bookmark_table = """
        CREATE TABLE IF NOT EXISTS bookmark (
        id INTEGER PRIMARY KEY AUTOINCREMENT,
        name TEXT NOT NULL UNIQUE,
        prompt TEXT NOT NULL,
        prepared_prompt TEXT NOT NULL,
        response TEXT NOT NULL,
        context_strategy TEXT,
        prompt_template TEXT,
        attack_module TEXT,
        metric TEXT,
        bookmark_time DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP
        );
    """

    sql_insert_bookmark_record = """
        INSERT INTO bookmark (
        name, prompt, prepared_prompt, response, context_strategy, prompt_template, attack_module,
        metric, bookmark_time)
        VALUES (?,?,?,?,?,?,?,?,?);
    """

    sql_select_bookmarks_record = """
        SELECT * FROM bookmark;
    """

    sql_select_bookmark_record = """
        SELECT * FROM bookmark WHERE name = ? ;
    """

    sql_delete_bookmark_records = """
        DELETE FROM bookmark;
    """

    def __init_instance(self, db_name) -> None:
        """
        Initialize the database instance for the Bookmark class.

        Args:
            db_name (str): The name of the database.
        """
        self.db_instance = Storage.create_database_connection(
            EnvVariables.BOOKMARKS.name, db_name, "db"
        )
        Storage.create_database_table(
            self.db_instance, Bookmark.sql_create_bookmark_table
        )

    def add_bookmark(self, bookmark: BookmarkArguments) -> dict:
        """
        Add a new bookmark to the database and return the success status.

        Args:
            bookmark (BookmarkArguments): The bookmark data to add.

        Returns:
            bool: True if the bookmark was added successfully, False otherwise.
        """
        bookmark.bookmark_time = datetime.now().replace(microsecond=0).isoformat(" ")

        data = (
            bookmark.name,
            bookmark.prompt,
            bookmark.prepared_prompt,
            bookmark.response,
            bookmark.context_strategy,
            bookmark.prompt_template,
            bookmark.attack_module,
            bookmark.metric,
            bookmark.bookmark_time,
        )
        try:
            results = Storage.create_database_record(
                self.db_instance, data, Bookmark.sql_insert_bookmark_record
            )
            if results is not None:
                return {"success": True, "message": "Bookmark added successfully."}
            else:
                raise Exception("Error inserting record into database.")
        except Exception as e:
            error_message = f"Failed to add bookmark record: {e}"
            return {"success": False, "message": error_message}

    def get_all_bookmarks(self) -> list[dict]:
        """
        Retrieve all bookmarks from the database.

        Returns:
            list[dict]: A list of all bookmarks as dictionaries.
        """
        list_of_bookmarks_tuples = Storage.read_database_records(
            self.db_instance,
            Bookmark.sql_select_bookmarks_record,
        )
        if list_of_bookmarks_tuples:
            list_of_bookmarks = [
                BookmarkArguments.from_tuple_to_dict(bookmark_tuple)
                for bookmark_tuple in list_of_bookmarks_tuples
            ]
        else:
            list_of_bookmarks = []
        return list_of_bookmarks

    def get_bookmark(self, bookmark_name: str) -> dict:
        """
        Retrieve a bookmark by its unique name.

        Args:
            bookmark_name (int): The unique name for the bookmark.

        Returns:
            dict: The bookmark information as a dictionary.

        Raises:
            RuntimeError: If the bookmark cannot be found.
        """
        if bookmark_name is not None:
            bookmark_info = Storage.read_database_record(
                self.db_instance, (bookmark_name,), Bookmark.sql_select_bookmark_record
            )
            if bookmark_info is not None:
                return BookmarkArguments.from_tuple_to_dict(bookmark_info)
            else:
                raise RuntimeError(
                    f"[Bookmark] No record found for bookmark name {bookmark_name}"
                )
        else:
            raise RuntimeError(f"[Bookmark] Invalid bookmark name: {bookmark_name}")

    def delete_bookmark(self, bookmark_name: str) -> dict:
        """
        Delete a bookmark by its unique name.

        Args:
            bookmark_name (str): The unique name for the bookmark to be deleted.
        """
        if bookmark_name is not None:
            try:
                sql_delete_bookmark_record = f"""
                    DELETE FROM bookmark WHERE name = '{bookmark_name}';
                """
                Storage.delete_database_record_in_table(
                    self.db_instance, sql_delete_bookmark_record
                )
                return {"success": True, "message": "Bookmark record deleted."}
            except Exception as e:
                error_message = f"Failed to delete bookmark record: {e}"
                return {"success": False, "message": error_message}
        else:
            error_message = f"[Bookmark] Invalid bookmark name: {bookmark_name}"
            return {"success": False, "message": error_message}

    def delete_all_bookmark(self) -> dict:
        """
        Delete all bookmarks from the database and return the operation result.

        Returns:
            dict: A dictionary with 'success' status and 'message' containing an error message if failed.
        """
        try:
            Storage.delete_database_record_in_table(
                self.db_instance, Bookmark.sql_delete_bookmark_records
            )
            return {"success": True, "message": "All bookmark records deleted."}
        except Exception as e:
            error_message = f"Failed to delete all bookmark records: {e}"
            return {"success": False, "message": error_message}

    def export_bookmarks(self, export_file_name: str = "bookmarks") -> str:
        """
        Export all bookmarks to a JSON file.

        This method retrieves all bookmarks from the database, converts them to a JSON format,
        and writes them to a file in the 'moonshot-data/bookmark' directory with the provided file name.

        Args:
            export_file_name (str): The base name of the file to export the bookmarks to.
                                    The '.json' extension will be appended to this base name.

        Returns:
            str: The path to the exported JSON file containing the bookmarks.
        """
        list_of_bookmarks_tuples = Storage.read_database_records(
            self.db_instance,
            Bookmark.sql_select_bookmarks_record,
        )

        if list_of_bookmarks_tuples is not None:
            bookmarks_json = [
                BookmarkArguments.from_tuple_to_dict(bookmark_tuple)
                for bookmark_tuple in list_of_bookmarks_tuples
            ]
        else:
            bookmarks_json = []

        try:
            return Storage.create_object(
                EnvVariables.BOOKMARKS.name,
                export_file_name,
                {"bookmarks": bookmarks_json},
                "json",
            )
        except Exception as e:
            logger.error(f"Failed to export bookmarks - {str(e)}.")
            raise e

    def close(self) -> None:
        """
        Close the database connection.
        """
        if self.db_instance:
            Storage.close_database_connection(self.db_instance)



import cmd2
from rich.console import Console
from rich.table import Table

from moonshot.api import api_delete_result, api_get_all_result, api_read_result
from moonshot.integrations.cli.benchmark.cookbook import show_cookbook_results
from moonshot.integrations.cli.benchmark.recipe import show_recipe_results
from moonshot.integrations.cli.common.display_helper import (
    display_view_list_format,
    display_view_str_format,
)
from moonshot.src.utils.find_feature import find_keyword

console = Console()


# ------------------------------------------------------------------------------
# CLI Functions
# ------------------------------------------------------------------------------
def list_results(args) -> list | None:
    """
    List all available results.

    This function retrieves all available results by calling the api_get_all_result_name function from the
    moonshot.api module. It then creates a table with the result id and name. If there are no results, it prints a
    message indicating that no results were found.

    Args:
        args: A namespace object from argparse. It should have an optional attribute:
        find (str): Optional field to find result(s) with a keyword.

    Returns:
        list | None: A list of Result or None if there is no result.
    """
    try:
        results_list = api_get_all_result()
        keyword = args.find.lower() if args.find else ""
        if keyword:
            filtered_results_list = find_keyword(keyword, results_list)
            if filtered_results_list:
                display_results(filtered_results_list)
                return filtered_results_list
            else:
                print("No results containing keyword found.")
                return None
        else:
            display_results(results_list)
            return results_list
    except Exception as e:
        print(f"[list_results]: {str(e)}")


def view_result(args) -> None:
    """
    View a specific result.

    This function retrieves a specific result by calling the api_read_result function from the
    moonshot.api module using the result filename provided in the args.
    It then checks if the result filename starts with "cookbook". If it does, it displays the result using the
    display_view_cookbook_result function. Otherwise, it uses the display_view_recipe_result function.

    Args:
        args: A namespace object from argparse. It should have the following attribute:
            result_filename (str): The filename of the result to view.

    Returns:
        None
    """
    try:
        result_info = api_read_result(args.result_filename)
        if result_info["metadata"].get("cookbooks"):
            display_view_cookbook_result(result_info)
        elif result_info["metadata"].get("recipes"):
            display_view_recipe_result(result_info)
        else:
            print("[view_result]: Unable to determine cookbook or recipe")
    except Exception as e:
        print(f"[view_result]: {str(e)}")


def delete_result(args) -> None:
    """
    Delete a result.

    This function deletes a result with the specified identifier. It prompts the user for confirmation before proceeding
    with the deletion. If the user confirms, it calls the api_delete_result function from the moonshot.api module to
    delete the result. If the deletion is successful, it prints a confirmation message. If an exception occurs, it
    prints an error message.

    Args:
        args: A namespace object from argparse. It should have the following attribute:
            result (str): The identifier of the result to delete.

    Returns:
        None
    """
    # Confirm with the user before deleting a result
    confirmation = console.input(
        "[bold red]Are you sure you want to delete the result (y/N)? [/]"
    )
    if confirmation.lower() != "y":
        console.print("[bold yellow]Result deletion cancelled.[/]")
        return
    try:
        api_delete_result(args.result)
        print("[delete_result]: Result deleted.")
    except Exception as e:
        print(f"[delete_result]: {str(e)}")


# ------------------------------------------------------------------------------
# Helper functions: Display on cli
# ------------------------------------------------------------------------------
def display_results(results_list):
    """
    Display a list of results.

    This function takes a list of results and displays them in a table format. If the list is empty, it prints a
    message indicating that no results were found.

    Args:
        results_list (list): A list of results. Each result is a dictionary with keys 'id' and 'name'.

    Returns:
        None
    """
    if results_list:
        table = Table(
            title="List of Results", show_lines=True, expand=True, header_style="bold"
        )
        table.add_column("No.", width=2)
        table.add_column("Result", justify="left", width=78)
        table.add_column("Contains", justify="left", width=20, overflow="fold")
        for result_id, result in enumerate(results_list, 1):
            metadata, results = result.values()

            id = metadata["id"]
            start_time = metadata["start_time"]
            end_time = metadata["end_time"]
            duration = metadata["duration"]
            status = metadata["status"]
            recipes = metadata["recipes"]
            cookbooks = metadata["cookbooks"]
            endpoints = metadata["endpoints"]
            num_of_prompts = metadata["num_of_prompts"]
            random_seed = metadata["random_seed"]
            system_prompt = metadata["system_prompt"]

            duration_info = (
                f"[blue]Period:[/blue] {start_time} - {end_time} ({duration}s)"
            )
            status_info = display_view_str_format("Status", status)
            recipes_info = display_view_list_format("Recipes", recipes)
            cookbooks_info = display_view_list_format("Cookbooks", cookbooks)
            endpoints_info = display_view_list_format("Endpoints", endpoints)
            prompts_info = display_view_str_format("Number of Prompts", num_of_prompts)
            seed_info = display_view_str_format("Seed", random_seed)
            system_prompt_info = display_view_str_format("System Prompt", system_prompt)

            result_info = f"[red]id: {id}[/red]\n\n{duration_info}\n\n{status_info}"
            contains_info = (
                f"{recipes_info}\n\n{cookbooks_info}\n\n{endpoints_info}\n\n{prompts_info}"
                f"\n\n{seed_info}\n\n{system_prompt_info}"
            )

            table.add_section()
            table.add_row(str(result_id), result_info, contains_info)
        console.print(table)
    else:
        console.print("[red]There are no results found.[/red]")


def display_view_recipe_result(result_info):
    """
    Display the recipe result.

    This function takes the result file and result info as arguments. It converts the result info into a dictionary
    using the convert_string_tuples_in_dict function. It then retrieves the recipes, endpoints, and duration from the
    converted result info. Finally, it calls the show_recipe_results function from the
    moonshot.integrations.cli.benchmark.recipe module to display the recipe results.

    Args:
        result_info (dict): The result info.

    Returns:
        None
    """
    recipes = result_info["metadata"]["recipes"]
    endpoints = result_info["metadata"]["endpoints"]
    duration = result_info["metadata"]["duration"]
    show_recipe_results(recipes, endpoints, result_info, duration)


def display_view_cookbook_result(result_info):
    """
    Display the cookbook result.

    This function takes the result file and result info as arguments. It converts the result info into a dictionary
    using the convert_string_tuples_in_dict function. It then retrieves the cookbooks, endpoints, and duration from the
    converted result info. Finally, it calls the show_cookbook_results function from the
    moonshot.integrations.cli.benchmark.cookbook module to display the cookbook results.

    Args:
        result_info (dict): The result info.

    Returns:
        None
    """
    cookbooks = result_info["metadata"]["cookbooks"]
    endpoints = result_info["metadata"]["endpoints"]
    duration = result_info["metadata"]["duration"]
    show_cookbook_results(cookbooks, endpoints, result_info, duration)


# ------------------------------------------------------------------------------
# Cmd2 Arguments Parsers
# ------------------------------------------------------------------------------
# View result arguments
view_result_args = cmd2.Cmd2ArgumentParser(
    description="View a result file.",
    epilog="Example:\n view_result my-new-cookbook-runner",
)
view_result_args.add_argument(
    "result_filename", type=str, help="Name of the result file"
)

# Delete result arguments
delete_result_args = cmd2.Cmd2ArgumentParser(
    description="Delete a result.",
    epilog="Example:\n delete_result my-new-cookbook-runner",
)
delete_result_args.add_argument("result", type=str, help="Name of the result")

# List result arguments
list_results_args = cmd2.Cmd2ArgumentParser(
    description="List all results.",
    epilog='Example:\n list_results -f "my-runner"',
)

list_results_args.add_argument(
    "-f",
    "--find",
    type=str,
    help="Optional field to find result(s) with keyword",
    nargs="?",
)
